package Philosophers is

   --  a circular table with a fixed number of chairs
   Chairs     : constant Positive := 5;
   --  and a position index type in the range 0 .. (Chairs-1)
   type Position_T is mod Chairs;

   --  at most (Chair-1) diners may dine together
   Max_Eaters : constant Natural  := Chairs - 1;
   subtype Eaters_T is Natural range 0 .. Max_Eaters;

   task type Philosopher_T
     (My_Position : Position_T;
      Think_Load  : Positive;
      Some_Time   : Positive) is
   end Philosopher_T;
   type Philosopher_Ref is access Philosopher_T;

   task type Forks_Control_T is
      entry Take_Fork;
      entry Put_Fork;
   end Forks_Control_T;
   --  possession of each fork is simulated by a mutually exclusive
   --  entry service provided by a Forks_Control server task in the
   --  corresponding position
   Forks : array (Position_T) of Forks_Control_T;

   --  any diner wanting to pick forks must bracket the attempt
   --  by mutually exclusive calls to the Deadlock_Preventer server
   --  task, which will only accept them as long as deadlock cannot
   --  occur
   task Deadlock_Preventer is
      entry Enter_Region;
      entry Leave_Region;
   end Deadlock_Preventer;

end Philosophers;
